<?php
namespace Bugfix\PatchWorkspaceimage\Utility;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Frontend\Page\PageRepository;

/**
 * A utility resolving and Caching the Rootline generation
 */
class RootlineUtility extends \TYPO3\CMS\Core\Utility\RootlineUtility
{

    /**
     * Resolve relations as defined in TCA and add them to the provided $pageRecord array.
     *
     * @param int $uid Page id
     * @param array $pageRecord Array with page data to add relation data to.
     * @throws \RuntimeException
     * @return array $pageRecord with additional relations
     */
    protected function enrichWithRelationFields($uid, array $pageRecord)
    {
        $pageOverlayFields = GeneralUtility::trimExplode(',', $GLOBALS['TYPO3_CONF_VARS']['FE']['pageOverlayFields']);
        foreach ($GLOBALS['TCA']['pages']['columns'] as $column => $configuration) {
            if ($this->columnHasRelationToResolve($configuration)) {
                $configuration = $configuration['config'];
                if ($configuration['MM']) {
                    /** @var $loadDBGroup \TYPO3\CMS\Core\Database\RelationHandler */
                    $loadDBGroup = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Database\RelationHandler::class);
                    $loadDBGroup->start(
                        $pageRecord[$column],
                        isset($configuration['allowed']) ? $configuration['allowed'] : $configuration['foreign_table'],
                        $configuration['MM'],
                        $uid,
                        'pages',
                        $configuration
                    );
                    $relatedUids = isset($loadDBGroup->tableArray[$configuration['foreign_table']])
                        ? $loadDBGroup->tableArray[$configuration['foreign_table']]
                        : [];
                } else {
                    $columnIsOverlaid = in_array($column, $pageOverlayFields, true);
                    $table = $configuration['foreign_table'];
                    $field = $configuration['foreign_field'];
                    $whereClauseParts = [$field . ' = ' . (int)($columnIsOverlaid ? $uid : $pageRecord['uid'])];
                    if (isset($configuration['foreign_match_fields']) && is_array($configuration['foreign_match_fields'])) {
                        foreach ($configuration['foreign_match_fields'] as $field => $value) {
                            $whereClauseParts[] = $field . ' = ' . $this->databaseConnection->fullQuoteStr($value, $table);
                        }
                    }
                    if (isset($configuration['foreign_table_field'])) {
                        if ((int)$this->languageUid > 0 && $columnIsOverlaid) {
                            $whereClauseParts[] = trim($configuration['foreign_table_field']) . ' = \'pages_language_overlay\'';
                        } else {
                            $whereClauseParts[] = trim($configuration['foreign_table_field']) . ' = \'pages\'';
                        }
                    }
                    if (isset($GLOBALS['TCA'][$table]['ctrl']['enablecolumns']['disabled'])) {
                        $whereClauseParts[] = $table . '.' . $GLOBALS['TCA'][$table]['ctrl']['enablecolumns']['disabled'] . ' = 0';
                    }
                    $whereClause = implode(' AND ', $whereClauseParts);
                    $whereClause .= $this->pageContext->deleteClause($table);
                    $whereClause .= $this->pageContext->enableFields($table); // neu FHO +++
                    $orderBy = isset($configuration['foreign_sortby']) ? $configuration['foreign_sortby'] : '';
                    $rows = $this->databaseConnection->exec_SELECTgetRows('uid', $table, $whereClause, '', $orderBy);

                    if (!is_array($rows)) {
                        throw new \RuntimeException('Could to resolve related records for page ' . $uid . ' and foreign_table ' . htmlspecialchars($configuration['foreign_table']), 1343589452);
                    }
                    $relatedUids = [];
                    foreach ($rows as $row) {
                        $relatedUids[] = $row['uid'];
                    }
                }
                $pageRecord[$column] = implode(',', $relatedUids);
            }
        }
        return $pageRecord;
    }
}

